//--------------------------------------------------------------
// File     : stm32_ub_udp_server.c
// Datum    : 07.02.2016
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F746
// IDE      : OpenSTM32
// GCC      : 4.9 2015q2
// Module   : CubeHAL
// Funktion : Ethernet UDP-Server (per PHY : LAN8742A-CZ)
//            IP-Stack = lwIP (V:1.4.1)
//
// Hinweis  : benutzt wird die RMII-Schnittstelle
//
//            PA1  = RMII_Ref_Clk       PC1 = ETH_MDC
//            PA2  = ETH_MDIO           PC4 = RMII_RXD0
//            PA7  = RMII_CRS_DV        PC5 = RMII_RXD1
//            PG11 = RMII_TX_EN
//            PG13 = RMII_TXD0          NRST = Reset
//            PG14 = RMII_TXD1
//
//--------------------------------------------------------------
// set include path to :
//   ${ProjDirPath}/udp_server/stm_lolevel
//   ${ProjDirPath}/udp_server/lwip_lolevel/system
//   ${ProjDirPath}/udp_server/lwip_lolevel/src/include
//   ${ProjDirPath}/udp_server/lwip_lolevel/src/include/ipv4
// set source location to :
//   /udp_server
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_udp_server.h"


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
uint8_t EthLinkStatus;
struct udp_pcb *upcb_server;
struct ip_addr OwnIPaddr;
struct ip_addr HostIPaddr;
extern struct netif gnetif;

//--------------------------------------------------------------
// Globale Struktur fr UDP_Server
//--------------------------------------------------------------
UDP_SERVER_t UDP_SERVER = {
  UDP_SERVER_NOINIT,"\0",0
};


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void udp_receive_callback(void *arg, struct udp_pcb *upcb, struct pbuf *p, struct ip_addr *addr, u16_t port);



//--------------------------------------------------------------
// Init vom UDP Server
// Return wert :
//      UDP_INIT_OK => Server wurde initialisiert
//   != UDP_INIT_OK => Fehler, Server wurde nicht initialisiert
//--------------------------------------------------------------
UDP_INIT_STATUS_t UB_UDP_Server_Init(void)
{
  UDP_INIT_STATUS_t ret_wert=UDP_INIT_OK;

  if(UDP_SERVER.status==UDP_SERVER_NOINIT) {
    EthLinkStatus=0;
    // Ethernet + LWIP-Stack init
    MX_LWIP_Init();

    // NVIC init
    HAL_NVIC_SetPriorityGrouping(NVIC_PRIORITYGROUP_4);
    HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);

    // Notify user
    User_notification(&gnetif);

    // init vom Server ist fertig
    UDP_SERVER.status=UDP_SERVER_NOCONNECT;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// zum starten vom UDP-Server
// Return wert :
//      UDP_CONNECT_OK => UDP-Server luft
//   != UDP_CONNECT_OK => Fehler, UDP-Server luft nicht
//--------------------------------------------------------------
UDP_SERVER_CONNECT_t UB_UDP_Server_Connect(void)
{
  UDP_SERVER_CONNECT_t ret_wert=UDP_CONNECT_OK;
  err_t err;

  if(UDP_SERVER.status==UDP_SERVER_NOINIT) return(UDP_INIT_ERR);
  if(UDP_SERVER.status==UDP_SERVER_RUNNING) return(UDP_RUNNING);
  if(EthLinkStatus>0) return(UDP_NO_LINK);

  // Verbindung fr den Server erstellen
  upcb_server = udp_new();

  if (upcb_server==NULL) return(UDP_ERR1);

  IP4_ADDR( &HostIPaddr, HOST_IP_ADDR0, HOST_IP_ADDR1, HOST_IP_ADDR2, HOST_IP_ADDR3 );
  IP4_ADDR( &OwnIPaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3 );

  // Server an eigene IP binden
  err=udp_bind(upcb_server,&OwnIPaddr,OWN_UDP_PORT);

  if (err != ERR_OK) return(UDP_ERR2);

  // Funktion fr UDP-Receive einrichten
  udp_recv(upcb_server, udp_receive_callback, NULL);

  // Connect vom Server ist fertig
  UDP_SERVER.status=UDP_SERVER_RUNNING;

  return(ret_wert);
}


//--------------------------------------------------------------
// zum bennden vom UDP-Server
//--------------------------------------------------------------
void UB_UDP_Server_Disconnect(void)
{
  if(UDP_SERVER.status==UDP_SERVER_RUNNING) {
    udp_remove(upcb_server);
    UDP_SERVER.status=UDP_SERVER_NOCONNECT;
  }
}


//--------------------------------------------------------------
// Sendet einen String per UDP zum Host
// Return_wert :
//  -> ERROR   , wenn String nicht gesendet wurde
//  -> SUCCESS , wenn String gesendet wurde
//--------------------------------------------------------------
ErrorStatus UB_UDP_Server_SendString(char *ptr)
{
  ErrorStatus ret_wert=ERROR;
  struct pbuf *p;
  err_t err;

  // nur senden, wenn Server luft
  if(UDP_SERVER.status==UDP_SERVER_RUNNING) {
    // Puffer einrichten
    p = pbuf_alloc(PBUF_TRANSPORT,strlen((char*)ptr), PBUF_POOL);
    if (p == NULL) return(ERROR);

    // String in Puffer kopieren
    pbuf_take(p, (char*)ptr, strlen((char*)ptr));

    // Daten per UDP an HOST senden
    err=udp_sendto(upcb_server,p,&HostIPaddr,HOST_UDP_PORT);

    // Puffer wieder lschen
    pbuf_free(p);

    if(err==ERR_OK) {
      ret_wert=SUCCESS;
    }
    else {
      udp_remove(upcb_server);
      UDP_SERVER.status=UDP_SERVER_NOCONNECT;
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// UDP Server
// diese Funktion muss zyklisch aufgerufen werden
// Return Wert :
//  -> wenn Server Offline   = UDP_SERVER_OFFLINE
//  -> wenn nichts empfangen = UDP_REVEICE_EMPTY
//  -> wenn String empfangen = UDP_RECEIVE_READY
//                             (-> String steht in *ptr)
//--------------------------------------------------------------
UDP_RECEIVE_t UB_UDP_Server_Do(char *ptr)
{
  UDP_RECEIVE_t ret_wert=UDP_REVEICE_EMPTY;
  uint32_t n;
  char wert;

  if(UDP_SERVER.status!=UDP_SERVER_NOINIT) {
    // LwIP zyklisch aufrufen
    ethernetif_input(&gnetif);
    sys_check_timeouts();

    // check ob LAN-Verbinguns noch ok
    if(EthLinkStatus>0) {
      if(UDP_SERVER.status==UDP_SERVER_RUNNING) {
        udp_remove(upcb_server);
        UDP_SERVER.status=UDP_SERVER_NOCONNECT;
      }
    }
  }

  if(UDP_SERVER.status!=UDP_SERVER_RUNNING) return(UDP_SERVER_OFFLINE);

  // Check ob ein UDP-String empfangen wurde
  if(UDP_SERVER.rxlen>0) {
    ret_wert=UDP_RECEIVE_READY;
    // String aus dem Puffer kopieren
    n=0;
    do {
      wert=UDP_SERVER.rxbuf[n];
      ptr[n]=wert;
      n++;
    }while((wert!=0) && (n<UDP_RX_BUFFER_SIZE-1));
    ptr[n]=0x00;
    // Merker wieder lschen
    UDP_SERVER.rxlen=0;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Funktion zum empfangen von
// UDP-Daten
// (wird automatisch beim UDP-receive aufgerufen)
//--------------------------------------------------------------
void udp_receive_callback(void *arg, struct udp_pcb *upcb, struct pbuf *p, struct ip_addr *addr, u16_t port)
{
  uint32_t n;
  char wert;
  char *ptr;

  ptr=(char*)p->payload;

  // empfangene Daten in Puffer kopieren
  n=0;
  do {
    wert=*ptr;
    UDP_SERVER.rxbuf[n]=wert;
    ptr++;
    n++;
  }while((wert!=0) && (n<UDP_RX_BUFFER_SIZE-1));
  UDP_SERVER.rxbuf[n]=0x00;
  // Merker setzen, das UDP-Daten vorhanden sind
  UDP_SERVER.rxlen=n;

  // Puffer lschen
  pbuf_free(p);
}

//--------------------------------------------------------------
void User_notification(struct netif *netif)
{
  if (netif_is_up(netif)) {
    // success
  }
  else {
    // error
  }
}

//--------------------------------------------------------------
void ethernetif_notify_conn_changed(struct netif *netif)
{
  if (netif_is_link_up(netif)) {
    // link up
    netif_set_up(netif);
  }
  else {
    // link down
    netif_set_down(netif);
  }
}





